@@ -26,7 +26,7 @@ except ImportError:
 
 class LpaDiagnostics( OpenPMDTimeSeries ):
 
-    def __init__( self, path_to_dir, check_all_files=True ):
+    def __init__( self, path_to_dir, check_all_files=True, backend=None ):
         """
         Initialize an OpenPMD time series with various methods to diagnose the
         data
@@ -44,9 +44,98 @@ class LpaDiagnostics( OpenPMDTimeSeries ):
             (i.e. that they contain the same fields and particles,
             with the same metadata)
             For fast access to the files, this can be changed to False.
+
+        backend: string
+            Backend to be used for data reading. Can be `openpmd-api`
+            or `h5py`. If not provided will use `openpmd-api` if available
+            and `h5py` otherwise.
         """
         OpenPMDTimeSeries.__init__( self, path_to_dir,
-                                    check_all_files=check_all_files )
+                                    check_all_files=check_all_files, backend=backend )
+
+    def get_energy_spread( self, t=None, iteration=None, species=None,
+                        select=None, center='mean', width='std', property='energy' ):
+        """
+        Calculate the central energy and energy spread according to the
+        particle weights
+
+        Parameters
+        ----------
+        t : float (in seconds), optional
+            Time at which to obtain the data (if this does not correspond to
+            an available file, the last file before `t` will be used)
+            Either `t` or `iteration` should be given by the user.
+
+        iteration : int
+            The iteration at which to obtain the data
+            Either `t` or `iteration` should be given by the user.
+
+        species : str
+            Particle species to use for calculations
+
+        select : dict, optional
+            Either None or a dictionary of rules
+            to select the particles, of the form
+            'x' : [-4., 10.]   (Particles having x between -4 and 10 meters)
+            'z' : [0, 100] (Particles having z between 0 and 100 meters)
+
+        center : str
+            Method to find the central energy of the particle distribution
+            Should be one of
+
+            - 'mean'
+            - 'median'
+
+        width : str
+            Method to find the energy spread of the particle distribution
+            Should be one of
+
+            - 'std'
+            - 'mad'
+
+        property : str
+            Unit of energy. Should be one of
+
+            - 'energy' returns energy in MeV
+            - 'gamma' returns Lorentz factor
+
+        Returns
+        -------
+        A tuple of floats with:
+        - central energy
+        - energy spread
+        """
+        # Get particle data
+        ux, uy, uz, w, m = self.get_particle(
+            var_list=['ux', 'uy', 'uz', 'w', 'mass'], select=select,
+            species=species, t=t, iteration=iteration)
+        # Calculate Lorentz factor and energy for all particles
+        gamma = np.sqrt(1 + ux ** 2 + uy ** 2 + uz ** 2)
+        if property == 'energy':
+            prop = (gamma - 1) * m * const.c ** 2 / const.e * 1e-6
+
+        elif property == 'gamma':
+            prop = gamma
+        else:
+            raise ValueError('Invalid output property: %s'%property)
+
+        # Calculate weighted center
+        if center == 'mean':
+            p_center = w_ave(prop, w)
+        elif center == 'median':
+            p_center = w_median(prop, w)
+        else:
+            raise ValueError('Invalid center property: %s' % center)
+
+        # Calculate weighted width
+        if width == 'std':
+            p_width = w_std(prop, w)
+        elif width == 'mad':
+            p_width = w_mad(prop, w)
+        else:
+            raise ValueError('Invalid width property: %s' % width)
+
+        return p_center, p_width
 
     def get_mean_gamma( self, t=None, iteration=None, species=None,
                         select=None ):
@@ -445,8 +534,9 @@ class LpaDiagnostics( OpenPMDTimeSeries ):
         # Return the current and bin centers
         return(current, info)
 
-    def get_laser_envelope( self, t=None, iteration=None, pol=None, m='all',
-                            theta=0, slice_across=None,
+    def get_laser_envelope( self, t=None, iteration=None, pol=None,
+                            laser_propagation='z',
+                            m='all', theta=0, slice_across=None,
                             slice_relative_position=None, plot=False,
                             plot_range=[[None, None], [None, None]], **kw ):
         """
@@ -465,7 +555,11 @@ class LpaDiagnostics( OpenPMDTimeSeries ):
             Either `t` or `iteration` should be given by the user.
 
         pol : string
-            Polarization of the field. Options are 'x', 'y'
+            Polarization of the field. Options are 'x', 'y', 'z'
+
+        laser_propagation : string, optional
+            Coordinate along which laser field propagates.
+            Default is 'z'.
 
         m : int or str, optional
            Only used for thetaMode geometry
@@ -515,44 +609,44 @@ class LpaDiagnostics( OpenPMDTimeSeries ):
         - A FieldMetaInformation object
         """
         # Check if polarization has been entered
-        if pol not in ['x', 'y']:
+        if pol not in ['x', 'y', 'z']:
             raise ValueError('The `pol` argument is missing or erroneous.')
 
-        # Prevent slicing across z, when extracting the raw electric field
-        # (z axis is needed for calculation of envelope)
-        # but record whether the user asked for slicing across z,
+        # Prevent slicing across `laser_propagation`, when extracting the raw electric field
+        # (`laser_propagation` axis is needed for calculation of envelope)
+        # but record whether the user asked for slicing across `laser_propagation`,
         # and whether a corresponding `slice_relative_position` coordinate
-        # along z was given, so as to perform this slicing later in this function.
-        slicing_coord_z = None
+        # along `laser_propagation` was given, so as to perform this slicing later in this function.
+        slicing_coord_laser = None
         if slice_across is not None:
             slice_across, slice_relative_position = \
                 sanitize_slicing(slice_across, slice_relative_position)
-            if 'z' in slice_across:
-                index_slicing_z = slice_across.index('z')
-                slice_across.pop(index_slicing_z)
-                slicing_coord_z = slice_relative_position.pop(index_slicing_z)
+            if laser_propagation in slice_across:
+                index_slicing_coord_laser = slice_across.index(laser_propagation)
+                slice_across.pop(index_slicing_coord_laser)
+                slicing_coord_laser = slice_relative_position.pop(index_slicing_coord_laser)
         # Get field data, and perform Hilbert transform
         field, info = self.get_field( t=t, iteration=iteration, field='E',
                               coord=pol, theta=theta, m=m,
                               slice_across=slice_across,
                               slice_relative_position=slice_relative_position )
-        e_complx = hilbert(field, axis=-1)
+        inverted_axes_dict = {info.axes[key]: key for key in info.axes.keys()}
+        e_complx = hilbert(field, axis=inverted_axes_dict[laser_propagation])
         envelope = np.abs(e_complx)
-        # If the user asked for slicing along z, do it now
-        if slicing_coord_z is not None:
-            inverted_axes_dict = {info.axes[key]: key for key in info.axes.keys()}
-            slicing_index = inverted_axes_dict['z']
-            coord_array = getattr( info, 'z' )
+        # If the user asked for slicing along `laser_propagation`, do it now
+        if slicing_coord_laser is not None:
+            slicing_index = inverted_axes_dict[laser_propagation]
+            coord_array = getattr( info, laser_propagation )
             # Number of cells along the slicing direction
             n_cells = len(coord_array)
             # Index of the slice (prevent stepping out of the array)
-            i_cell = int( 0.5 * (slicing_coord_z + 1.) * n_cells )
+            i_cell = int( 0.5 * (slicing_coord_laser + 1.) * n_cells )
             i_cell = max( i_cell, 0 )
             i_cell = min( i_cell, n_cells - 1)
             envelope = np.take( envelope, [i_cell], axis=slicing_index )
             envelope = np.squeeze(envelope)
             # Remove the sliced labels from the FieldMetaInformation
-            info._remove_axis('z')
+            info._remove_axis(laser_propagation)
 
         # Plot the result if needed
         if plot:
@@ -829,7 +923,7 @@ class LpaDiagnostics( OpenPMDTimeSeries ):
         # Find the indices of the maximum field, and
         # pick the corresponding transverse slice
         itrans_max, iz_max = np.unravel_index(
-            np.argmax( field ), dims=field.shape )
+            np.argmax( field ), field.shape )
         trans_slice = field[ :, iz_max ]
         # Get transverse positons
         trans_pos = getattr(info, info.axes[0])
@@ -1020,6 +1114,54 @@ def w_std( a, weights ):
         variance = np.average((a - average) ** 2, weights=weights)
         return( np.sqrt(variance) )
 
+def w_median(a, weights):
+    """
+    Compute the weighted median of a 1D numpy array.
+    Parameters
+    ----------
+    a : ndarray
+        Input array (one dimension).
+    weights : ndarray
+        Array with the weights of the same size of `data`.
+    Returns
+    -------
+    median : float
+        The output value.
+    """
+    quantile = .5
+    if not isinstance(a, np.matrix):
+        a = np.asarray(a)
+    if not isinstance(weights, np.matrix):
+        weights = np.asarray(weights)
+    if a.shape != weights.shape:
+        raise TypeError("the length of data and weights must be the same")
+    ind_sorted = np.argsort(a)
+    sorted_data = a[ind_sorted]
+    sorted_weights = weights[ind_sorted]
+
+    Sn = np.cumsum(sorted_weights)
+    # Center and normalize the cumsum (i.e. divide by the total sum)
+    Pn = (Sn - 0.5 * sorted_weights) / Sn[-1]
+    # Get the value of the weighted median
+    return np.interp(quantile, Pn, sorted_data)
+
+def w_mad(a, w):
+    """
+    Compute the weighted median absolute deviation of a 1D numpy array.
+    Parameters
+    ----------
+    a : ndarray
+        Input array (one dimension).
+    weights : ndarray
+        Array with the weights of the same size of `data`.
+    Returns
+    -------
+    mad : float
+        The output value.
+    """
+    med = w_median(a, w)
+    mad = w_median(np.abs(a - med), w)
+    return mad
 
 def gaussian_profile( x, x0, E0, w0 ):
     """
